# coding=utf-8
import os
import shutil
import tempfile
from unittest import TestCase

import mock
from mock import Mock

from earthdiagnostics.cmormanager import CMORManager
from earthdiagnostics.modelingrealm import ModelingRealms


class TestCMORManager(TestCase):

    def setUp(self):
        self.config = Mock()
        self.config.data_convention = 'specs'
        self.config.data_type = 'exp'
        self.config.experiment.expid = 'expid'
        self.config.experiment.model = 'model'
        self.config.experiment.experiment_name = 'expname'
        self.config.experiment.institute = 'institute'
        self.config.experiment.member_count_start = 0
        self.config.experiment.atmos_timestep = 6
        self.config.experiment.ocean_timestep = 6
        self.config.experiment.chunk_size = 12
        self.config.experiment.num_chunks = 1
        self.config.experiment.calendar = 'standard'

        self.config.cmor.initialization_number = 1
        self.config.cmor.version = ''
        self.config.cmor.default_ocean_grid = 'ocean_grid'
        self.config.cmor.default_atmos_grid = 'atmos_grid'
        self.config.cmor.activity = 'activity'
        self.config.cmor.force = False
        self.config.cmor.force_untar = False

        self.tmp_dir = tempfile.mkdtemp()
        os.mkdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        self.config.data_dir = self.tmp_dir

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_find_data(self):
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'expid'))

    def test_find_data_fail(self):
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        with self.assertRaises(Exception):
            CMORManager(self.config)

    def test_find_data_with_model(self):
        os.makedirs(os.path.join(self.tmp_dir, 'model', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'model', 'expid'))

    def test_find_data_with_ecearth_fix(self):
        self.config.experiment.model = 'EC-Earth'
        os.makedirs(os.path.join(self.tmp_dir, 'ecearth', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'ecearth', 'expid'))

    def test_find_data_with_type_and_model(self):
        os.makedirs(os.path.join(self.tmp_dir, 'exp', 'model', self.config.experiment.expid))
        os.rmdir(os.path.join(self.tmp_dir, self.config.experiment.expid))
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.cmor_path, os.path.join(self.tmp_dir, 'exp', 'model', 'expid'))

    def test_get_varfolder(self):
        cmor_manager = CMORManager(self.config)
        self.assertEqual(cmor_manager.get_varfolder(ModelingRealms.ocean, 'var'),
                         'var_f6h')

    def test_get_file_path_specs(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_version(self):
        self.config.cmor.version = 'version'
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/version/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_grid(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency, 'grid')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/grid/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_198901-198912.nc'))

    def test_get_file_path_specs_year(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'year'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'year'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, year='1998')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/year/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_1998.nc'))

        frequency.frequency = 'other'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'other'
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', None, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998')

    def test_get_file_path_raise_incomaptible_date_info(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'monthly'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'

        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, date_str='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          None, frequency, year='1998', date_str='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998', date_str='1998')
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          None, frequency)

    def test_get_file_path_specs_date_str(self):
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        frequency.frequency = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, date_str='date_str')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/institute/model/expname/S19900101/frequency/'
                                                    'ocean/var/r2i1p1/'
                                                    'var_Omon_model_expname_S19900101_r2i1p1_date_str.nc'))

    def test_get_file_path_primavera(self):
        self._configure_primavera()
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/activity/institute/model/expname/'
                                                    'r2i1p1f1/Omon/var/ocean_grid/version/'
                                                    'var_Omon_model_expname_r2i1p1f1_ocean_grid_198901-198912.nc'))

    def test_get_file_path_no_version_primavera(self):
        self._configure_primavera()
        self.config.cmor.version = ''
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var', cmor_var,
                          0, frequency)

    def _configure_primavera(self):
        self.config.data_convention = 'primavera'
        self.config.cmor.version = 'version'

    def _configure_meteofrance(self):
        self.config.data_convention = 'meteofrance'

    def test_get_file_path_primavera_grid(self):
        self._configure_primavera()
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, 0,
                                               frequency, 'grid')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/activity/institute/model/expname/r2i1p1f1/'
                                                    'Omon/var/grid/version/'
                                                    'var_Omon_model_expname_r2i1p1f1_grid_198901-198912.nc'))

    def test_get_file_path_primavera_year(self):
        self._configure_primavera()
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'year'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'year'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, year='1998')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/activity/institute/model/expname/r2i1p1f1/Omon/'
                                                    'var/ocean_grid/version/'
                                                    'var_Omon_model_expname_r2i1p1f1_ocean_grid_1998.nc'))

        frequency.frequency = 'other'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'other'
        self.assertRaises(ValueError, cmor_manager.get_file_path, '19900101', None, ModelingRealms.ocean, 'var', cmor_var,
                          1, frequency, year='1998')

    def test_get_file_path_primavera_date_str(self):
        self._configure_primavera()
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'
        frequency.frequency = 'frequency'
        file_path = cmor_manager.get_file_path('19900101', 1, ModelingRealms.ocean, 'var', cmor_var, None,
                                               frequency, date_str='date_str')
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/activity/institute/model/expname/r2i1p1f1/'
                                                    'Omon/var/ocean_grid/version/'
                                                    'var_Omon_model_expname_r2i1p1f1_ocean_grid_date_str.nc'))

    def test_file_exists(self):
        with mock.patch('os.path.isfile') as isfile:
            cmor_manager = CMORManager(self.config)
            isfile.return_value = True
            self.assertTrue(cmor_manager.file_exists(ModelingRealms.ocean, 'var', '20011101', 1,1))
            isfile.return_value = False
            self.assertFalse(cmor_manager.file_exists(ModelingRealms.ocean, 'var', '20011101', 1,1))


    def test_file_exists_multiple_versions(self):
        with mock.patch('os.path.isfile') as isfile:
            cmor_manager = CMORManager(self.config)
            isfile.return_value = True
            self.assertTrue(cmor_manager.file_exists(ModelingRealms.ocean, 'var', '20011101', 1,1,
                                                     possible_versions=('version1', 'version2')))
            isfile.return_value = False
            self.assertFalse(cmor_manager.file_exists(ModelingRealms.ocean, 'var', '20011101', 1,1,
                                                      possible_versions=('version1', 'version2')))

    def test_get_file_path_meteofrance(self):
        self._configure_meteofrance()
        cmor_manager = CMORManager(self.config)
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'day'
        file_path = cmor_manager.get_file_path('20110101', 1, ModelingRealms.ocean, 'soicecov', cmor_var, 1,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expname/HA/2011/soicecov_day_201101_01.nc'))

        file_path = cmor_manager.get_file_path('20110101', 1, ModelingRealms.ocean, 'soicecov', cmor_var, 2,
                                               frequency)
        self.assertEqual(file_path,
                         os.path.join(self.tmp_dir, 'expname/HA/2011/soicecov_day_201201_01.nc'))

    def test_create_link(self):
        cmor_manager = CMORManager(self.config)
        file_descriptor, path = tempfile.mkstemp(dir=self.tmp_dir)
        os.close(file_descriptor)
        frequency = Mock()
        frequency.folder_name.return_value = 'frequency_folder'
        cmor_manager.create_link(ModelingRealms.ocean, path, frequency, 'var', None, False, None)
        filename = os.path.basename(path)
        self.assertTrue(os.path.islink(os.path.join(self.tmp_dir, 'expid', 'frequency_folder', 'var_f6h', filename)))

    def test_create_link_with_grid(self):
        cmor_manager = CMORManager(self.config)
        file_descriptor, path = tempfile.mkstemp(dir=self.tmp_dir)
        os.close(file_descriptor)
        frequency = Mock()
        frequency.folder_name.return_value = 'frequency_folder'
        cmor_manager.create_link(ModelingRealms.ocean, path, frequency, 'var', 'grid', False, None)
        filename = os.path.basename(path)
        var_mainfolder = os.path.join(self.tmp_dir, 'expid', 'frequency_folder', 'var_f6h')
        var_grid_folder = os.path.join(self.tmp_dir, 'expid', 'frequency_folder', 'var-grid_f6h')
        self.assertTrue(os.path.islink(var_mainfolder))
        self.assertTrue(os.path.islink(os.path.join(var_grid_folder, filename)))
        self.assertTrue(os.readlink(var_mainfolder), var_grid_folder)

    @mock.patch('earthdiagnostics.cmormanager.Cmorizer', autospec=True)
    def test_prepare_cmorize(self, mock_cmor):
        mock_instance = mock_cmor.return_value
        cmor_manager = CMORManager(self.config)
        self.config.experiment.get_member_list.return_value = (('20000101', 2),)
        cmor_manager.prepare()
        mock_instance.cmorize_ocean.assert_called_once()
        mock_instance.cmorize_atmos.assert_called_once()

    @mock.patch('earthdiagnostics.cmormanager.Cmorizer', autospec=True)
    def test_prepare_cmorize_force(self, mock_cmor):
        mock_instance = mock_cmor.return_value
        self.config.cmor.force = True
        cmor_manager = CMORManager(self.config)
        self.config.experiment.get_member_list.return_value = (('20000101', 2),)
        cmor_manager.prepare()
        mock_instance.cmorize_ocean.assert_called_once()
        mock_instance.cmorize_atmos.assert_called_once()

    @mock.patch('earthdiagnostics.cmormanager.CMORManager.is_cmorized')
    @mock.patch('earthdiagnostics.cmormanager.CMORManager.create_links')
    @mock.patch('earthdiagnostics.utils.Utils.unzip')
    @mock.patch('earthdiagnostics.utils.Utils.untar')
    def test_prepare_cmorize_force_untar(self, mock_untar, mock_unzip, mock_create_links, mock_cmorized):
        original_cmor_path = os.path.join(self.config.data_dir, self.config.experiment.expid,
                                          'original_files', 'cmorfiles')
        os.makedirs(original_cmor_path)
        self.config.experiment.get_member_str.return_value = 'r1i1p1'
        self.config.experiment.get_chunk_start_str.return_value = '20000101'
        cmor_prefix = 'CMORT_{0}_{1}_{2}_{3}-'.format(self.config.experiment.expid, '20000101', 'r1i1p1',
                                                      '20000101')
        tempfile.mkstemp('.tar.gz', cmor_prefix, original_cmor_path)
        tempfile.mkstemp('.tar', cmor_prefix, original_cmor_path)
        mock_cmorized.return_value = True
        self.config.cmor.force_untar = True
        cmor_manager = CMORManager(self.config)
        self.config.experiment.get_member_list.return_value = (('20000101', 2),)
        cmor_manager.prepare()
        mock_create_links.assert_called_once()
        mock_unzip.assert_called_once()
        mock_untar.assert_called_once()

    @mock.patch('earthdiagnostics.cmormanager.CMORManager.is_cmorized')
    def test_prepare_already_cmorized(self, mock_cmorized):
        mock_cmorized.return_value = True
        self.config.cmor.force_untar = False
        cmor_manager = CMORManager(self.config)
        self.config.experiment.get_member_list.return_value = (('20000101', 2),)
        cmor_manager.prepare()

    @mock.patch('earthdiagnostics.cmormanager.CMORManager.is_cmorized')
    def test_prepare_cmorization_not_requested(self, mock_cmorized):
        mock_cmorized.return_value = False
        self.config.cmor.chunk_cmorization_requested.return_value = False
        cmor_manager = CMORManager(self.config)
        self.config.experiment.get_member_list.return_value = (('20000101', 2),)
        cmor_manager.prepare()

    def test_prepare_meteofrance(self):
        self._configure_meteofrance()
        cmor_manager = CMORManager(self.config)
        cmor_manager.prepare()

